const pbkdf2 = require('pbkdf2');
const MCrypt = require('mcrypt').MCrypt;

// Translated from PHP to JS
class PKCS7 {
  pad(blockSize, strLen) {
    const paddingLen = blockSize - (strLen % blockSize);
    const paddingChar = String.fromCharCode(paddingLen);

    return paddingChar.repeat(paddingLen);
  }

  unpad(blockSize, paddedStr) {
    const paddedStrLen = paddedStr.length;
    const pad = paddedStr.charCodeAt(paddedStrLen - 1);

    return paddedStr.substr(0, paddedStrLen - pad);
  }
}

const _PKCS7 = new PKCS7();

class Rijndael {
  constructor(options) {
    this.mcrypt = new MCrypt('rijndael-128', 'cbc');
    this.blockSize = this.mcrypt.getIvSize();

    this.setOptions({ iteration: 1000, digest: 'sha1', ...options });
  }

  setOptions(options) {
    this.options = { ...this.options, ...options };
  }

  createIv() {
    return pbkdf2.pbkdf2Sync(
      this.options.password,
      this.options.salt,
      this.options.iteration,
      this.blockSize,
      this.options.digest
    );
  }

  createKey() {
    return new Buffer(
      '86743bfa7182fa60dd15439f450dac65d154feb0616aafa0de150681e4c41fb0',
      'hex'
    );
  }

  encrypt(data) {
    this.mcrypt.open(this.createKey(), this.createIv());

    const encryptedData = this.mcrypt.encrypt(
      `${data}${_PKCS7.pad(this.blockSize, data.length)}`
    );
    return encryptedData.toString('base64');
  }

  decrypt(data) {
    this.mcrypt.open(this.createKey(), this.createIv());
    const decrypted = this.mcrypt.decrypt(new Buffer(data, 'base64'));
    const decodedDecryptedData = decrypted.toString();

    return _PKCS7.unpad(this.blockSize, decodedDecryptedData);
  }
}

module.exports = Rijndael;
